#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test creating a component with the library editor
"""


def test(library_editor, helpers):
    """
    Create new component
    """
    le = library_editor

    # Open "New Library Element" wizard
    le.action('libraryEditorActionNewElement').trigger(blocking=False)

    # Choose type of element
    le.widget('libraryEditorNewElementWizardChooseTypeComponentButton').click()

    # Enter metadata
    widget_properties = {
        ('NameEdit', 'text'): 'New Component',
        ('DescriptionEdit', 'plainText'): 'Foo Bar',
        ('KeywordsEdit', 'text'): '',
        ('AuthorEdit', 'text'): 'Functional Test',
        ('VersionEdit', 'text'): '1.2.3',
    }
    for (widget, property), value in widget_properties.items():
        le.widget('libraryEditorNewElementWizardMetadata' + widget).set_property(property, value)
    le.widget('libraryEditorNewElementWizardNextButton').click()

    # Enter component properties
    widget_properties = {
        ('SchematicOnlyComboBox', 'checked'): False,
        ('DefaultValueEdit', 'plainText'): 'Default Value',
        ('PrefixEdit', 'text'): 'X',
    }
    for (widget, property), value in widget_properties.items():
        le.widget('libraryEditorNewElementWizardComponentProperties' + widget).set_property(property, value)
    le.widget('libraryEditorNewElementWizardNextButton').click()

    # Add a symbol
    le.widget('libraryEditorNewElementWizardComponentSymbolsChooseSymbolButton').click()
    category_tree = le.widget('libraryEditorNewElementWizardComponentSymbolsChooseSymbolDialogCategoriesTree')
    helpers.wait_for_model_items_count(category_tree, 1)
    category = category_tree.model().items().items[0]
    category_tree.select_item(category)
    symbols_list = le.widget('libraryEditorNewElementWizardComponentSymbolsChooseSymbolDialogSymbolsList')
    helpers.wait_for_model_items_count(symbols_list, 1)
    symbol = symbols_list.model().items().items[0]
    symbols_list.select_item(symbol)
    le.widget('libraryEditorNewElementWizardComponentSymbolsChooseSymbolDialogAcceptButton').click()
    le.widget('libraryEditorNewElementWizardComponentSymbolsAddSymbolButton').click()
    le.widget('libraryEditorNewElementWizardNextButton').click()

    # Define signals
    # (Do nothing as signals are automatically generated)
    le.widget('libraryEditorNewElementWizardNextButton').click()

    # Define pin-signal-map with auto-assign
    le.widget('libraryEditorNewElementWizardComponentPinSignalMapAutoAssignButton').click()

    # Go back and forth the wizard pages to test if everything is memorized
    # properly. In the past, the wizard even crashed when doing this.
    le.widget('libraryEditorNewElementWizardBackButton').click()
    le.widget('libraryEditorNewElementWizardBackButton').click()
    le.widget('libraryEditorNewElementWizardBackButton').click()
    le.widget('libraryEditorNewElementWizardBackButton').click()
    le.widget('libraryEditorNewElementWizardNextButton').click()
    le.widget('libraryEditorNewElementWizardNextButton').click()
    le.widget('libraryEditorNewElementWizardNextButton').click()
    le.widget('libraryEditorNewElementWizardNextButton').click()

    # Finish
    dialog = le.widget('libraryEditorNewElementWizard')
    le.widget('libraryEditorNewElementWizardFinishButton').click()
    helpers.wait_until_widget_hidden(dialog)

    # Check if a new tab is opened (indicates that the element was created)
    tab_props = le.widget('libraryEditorStackedWidget').properties()
    assert tab_props['count'] == 2
    assert tab_props['currentIndex'] == 1

    # Check metadata
    assert le.widget('libraryEditorComponentNameEdit').properties()['text'] == 'New Component'
    assert le.widget('libraryEditorComponentDescriptionEdit').properties()['plainText'] == 'Foo Bar'
    assert le.widget('libraryEditorComponentKeywordsEdit').properties()['text'] == ''
    assert le.widget('libraryEditorComponentAuthorEdit').properties()['text'] == 'Functional Test'
    assert le.widget('libraryEditorComponentVersionEdit').properties()['text'] == '1.2.3'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
import os

"""
Test image export dialog in board editor

Meant as an addition to the tests in test_schematic_image_export.py just to
test the board editor specific functionality works as well.
"""


@pytest.fixture
def dialog(project_editor):
    """
    Fixture opening the image export dialog in the board editor
    """
    project_editor.action('boardEditorActionExportImage').\
        trigger(blocking=False)
    dialog = project_editor.widget('boardEditorGraphicsExportDialog')

    # Do not open exported files since this would be annoying here.
    project_editor.widget(
        'boardEditorGraphicsExportDialogOpenFileCheckbox').\
        set_property('checked', False)

    yield dialog


def test_export_png(dialog, project_editor, librepcb, helpers):
    """
    Test if the accept button asks for a filepath and then creates a PNG file
    """
    project_editor.widget(
        'boardEditorGraphicsExportDialogButtonAccept').click()
    path = librepcb.abspath('test export.png')
    project_editor.widget(
        'boardEditorGraphicsExportSaveFileDialogFileNameEdit').\
        set_property('text', path)
    project_editor.widget(
        'boardEditorGraphicsExportSaveFileDialogOkButton').click()
    helpers.wait_until_widget_hidden(dialog)  # raises on timeout
    assert os.path.exists(path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test creating a package with the library editor
"""


def test(library_editor, helpers):
    """
    Create new package
    """
    le = library_editor

    # Open "New Library Element" wizard
    le.action('libraryEditorActionNewElement').trigger(blocking=False)

    # Choose type of element
    le.widget('libraryEditorNewElementWizardChooseTypePackageButton').click()

    # Enter metadata
    widget_properties = {
        ('NameEdit', 'text'): 'New Package',
        ('DescriptionEdit', 'plainText'): 'Foo Bar',
        ('KeywordsEdit', 'text'): '',
        ('AuthorEdit', 'text'): 'Functional Test',
        ('VersionEdit', 'text'): '1.2.3',
    }
    for (widget, property), value in widget_properties.items():
        le.widget('libraryEditorNewElementWizardMetadata' + widget).set_property(property, value)
    le.widget('libraryEditorNewElementWizardNextButton').click()

    # Finish
    dialog = le.widget('libraryEditorNewElementWizard')
    le.widget('libraryEditorNewElementWizardFinishButton').click()
    helpers.wait_until_widget_hidden(dialog)

    # Check if a new tab is opened (indicates that the element was created)
    tab_props = le.widget('libraryEditorStackedWidget').properties()
    assert tab_props['count'] == 2
    assert tab_props['currentIndex'] == 1

    # Check metadata
    assert le.widget('libraryEditorPackageNameEdit').properties()['text'] == 'New Package'
    assert le.widget('libraryEditorPackageDescriptionEdit').properties()['plainText'] == 'Foo Bar'
    assert le.widget('libraryEditorPackageKeywordsEdit').properties()['text'] == ''
    assert le.widget('libraryEditorPackageAuthorEdit').properties()['text'] == 'Functional Test'
    assert le.widget('libraryEditorPackageVersionEdit').properties()['text'] == '1.2.3'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest

"""
Test "add component"-dialog in project editor
"""


class AddComponentDialogHelper(object):
    def __init__(self, app):
        super(AddComponentDialogHelper, self).__init__()
        self.app = app
        self.app.action('schematicEditorActionAddComponent').trigger(blocking=False)
        self.dialog = self.app.widget('schematicEditorAddComponentDialog')


@pytest.fixture
def add_component_dialog(project_editor, helpers):
    """
    Fixture opening the "Add Component"-editor in the schematic editor
    """
    helpers.wait_for_library_scan_complete(project_editor)
    helper = AddComponentDialogHelper(project_editor)
    yield helper


def test_if_dialog_is_active_widget(add_component_dialog, helpers):
    """
    Test if the dialog is the active widget after opening it
    """
    helpers.wait_for_active_dialog(add_component_dialog.app, add_component_dialog.dialog)  # raises on timeout


def test_init_state(add_component_dialog, helpers):
    """
    Test if the initial state of various widgets is correct
    """
    # Category tree must be populated
    category_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogCategoryTree')
    helpers.wait_for_model_items_count(category_tree, 1)
    assert category_tree.model().items().items[0].value == 'Capacitors'

    # Component tree must be empty
    component_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentTree')
    helpers.wait_for_model_items_count(component_tree, 0, 0)

    # Symbol variant combobox must be hidden
    symbvar_combobox = add_component_dialog.app.widget('schematicEditorAddComponentDialogSymVarCombobox', wait_active=False)
    assert symbvar_combobox.properties()['visible'] is False


def test_if_cancel_closes_dialog(add_component_dialog, helpers):
    """
    Test if the cancel button closes the dialog
    """
    add_component_dialog.app.widget('schematicEditorAddComponentDialogButtonCancel').click()
    helpers.wait_until_widget_hidden(add_component_dialog.dialog)  # raises on timeout


def test_if_accept_does_nothing_if_no_component_selected(add_component_dialog):
    """
    Test if the accept button does nothing if no component is selected
    """
    add_component_dialog.app.widget('schematicEditorAddComponentDialogButtonAccept').click()
    assert add_component_dialog.dialog.properties()['visible'] is True


def test_select_component_with_one_symbvar(add_component_dialog, helpers):
    """
    Test selecting a component with only one symbol variant
    """
    # Select category
    category_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogCategoryTree')
    helpers.wait_for_model_items_count(category_tree, 3)
    category = category_tree.model().items().items[2]
    assert category.value == 'Diodes'
    category_tree.select_item(category)

    # Select component
    component_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentTree')
    helpers.wait_for_model_items_count(component_tree, 1)
    component = component_tree.model().items().items[0]
    assert component.value == 'Diode'
    component_tree.select_item(component)

    # Symbol variant must be hidden
    symbvar_combobox = add_component_dialog.app.widget('schematicEditorAddComponentDialogSymVarCombobox', wait_active=False)
    assert symbvar_combobox.properties()['visible'] is False

    # Check labels
    component_name_label = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentNameLabel')
    assert component_name_label.properties()['text'] == 'Diode'
    device_name_label = add_component_dialog.app.widget('schematicEditorAddComponentDialogDeviceNameLabel')
    assert device_name_label.properties()['text'] == 'No device selected'

    # Accept
    add_component_dialog.app.widget('schematicEditorAddComponentDialogButtonAccept').click()
    helpers.wait_until_widget_hidden(add_component_dialog.dialog)  # raises on timeout


def test_select_component_with_two_symbvars(add_component_dialog, helpers):
    """
    Test selecting a component with two symbol variants
    """
    # Select category
    category_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogCategoryTree')
    helpers.wait_for_model_items_count(category_tree, 1)
    category = category_tree.model().items().items[0]
    assert category.value == 'Capacitors'
    category_tree.select_item(category)

    # Select component
    component_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentTree')
    helpers.wait_for_model_items_count(component_tree, 1)
    component = component_tree.model().items().items[0]
    assert component.value == 'Capacitor Bipolar'
    component_tree.select_item(component)

    # Symbol variant must be visible
    symbvar_combobox = add_component_dialog.app.widget('schematicEditorAddComponentDialogSymVarCombobox')
    helpers.wait_for_model_items_count(symbvar_combobox, 2, 2)
    symbvar_items = symbvar_combobox.model().items()
    assert symbvar_items.items[0].value == 'European [IEC 60617]'
    assert symbvar_items.items[1].value == 'American [IEEE 315]'
    assert symbvar_combobox.properties()['currentText'] == symbvar_items.items[0].value

    # Check labels
    component_name_label = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentNameLabel')
    assert component_name_label.properties()['text'] == 'Capacitor Bipolar'
    device_name_label = add_component_dialog.app.widget('schematicEditorAddComponentDialogDeviceNameLabel')
    assert device_name_label.properties()['text'] == 'No device selected'

    # Accept
    add_component_dialog.app.widget('schematicEditorAddComponentDialogButtonAccept').click()
    helpers.wait_until_widget_hidden(add_component_dialog.dialog)  # raises on timeout


def test_select_device(add_component_dialog, helpers):
    """
    Test selecting a device
    """
    # Select category
    category_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogCategoryTree')
    helpers.wait_for_model_items_count(category_tree, 1)
    category = category_tree.model().items().items[0]
    assert category.value == 'Capacitors'
    category_tree.select_item(category)

    # Select device
    component_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentTree')
    helpers.wait_for_model_items_count(component_tree, 1)
    device = component_tree.model().items().items[0].items[1]
    assert device.value == 'C-0805'
    component_tree.select_item(device)

    # Symbol variant must be visible
    symbvar_combobox = add_component_dialog.app.widget('schematicEditorAddComponentDialogSymVarCombobox')
    helpers.wait_for_model_items_count(symbvar_combobox, 2, 2)
    symbvar_items = symbvar_combobox.model().items()
    assert symbvar_items.items[0].value == 'European [IEC 60617]'
    assert symbvar_items.items[1].value == 'American [IEEE 315]'
    assert symbvar_combobox.properties()['currentText'] == symbvar_items.items[0].value

    # Check labels
    component_name_label = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentNameLabel')
    assert component_name_label.properties()['text'] == 'Capacitor Bipolar'
    device_name_label = add_component_dialog.app.widget('schematicEditorAddComponentDialogDeviceNameLabel')
    assert device_name_label.properties()['text'] == 'C-0805'

    # Accept
    add_component_dialog.app.widget('schematicEditorAddComponentDialogButtonAccept').click()
    helpers.wait_until_widget_hidden(add_component_dialog.dialog)  # raises on timeout


def test_select_component_by_doubleclick(add_component_dialog, helpers):
    """
    Test selecting a component by double-clicking the tree item
    """
    # Select category
    category_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogCategoryTree')
    helpers.wait_for_model_items_count(category_tree, 3)
    category = category_tree.model().items().items[2]
    assert category.value == 'Diodes'
    category_tree.select_item(category)

    # Select component
    component_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentTree')
    helpers.wait_for_model_items_count(component_tree, 1)
    component = component_tree.model().items().items[0]
    assert component.value == 'Diode'
    component_tree.dclick_item(component)

    # Check if dialog is closed
    helpers.wait_until_widget_hidden(add_component_dialog.dialog)  # raises on timeout


def test_select_device_by_doubleclick(add_component_dialog, helpers):
    """
    Test selecting a device by double-clicking the tree item
    """
    # Select category
    category_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogCategoryTree')
    helpers.wait_for_model_items_count(category_tree, 1)
    category = category_tree.model().items().items[0]
    assert category.value == 'Capacitors'
    category_tree.select_item(category)

    # Select device
    component_tree = add_component_dialog.app.widget('schematicEditorAddComponentDialogComponentTree')
    helpers.wait_for_model_items_count(component_tree, 1)
    device = component_tree.model().items().items[0].items[1]
    assert device.value == 'C-0805'
    component_tree.dclick_item(device)

    # Check if dialog is closed
    helpers.wait_until_widget_hidden(add_component_dialog.dialog)  # raises on timeout



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest

"""
Test command "open-step"
"""


def test_valid_file(cli):
    fp = cli.add_file('unittests/librepcbcommon/OccModelTest/model.step')
    code, stdout, stderr = cli.run('open-step', fp)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == ''
    assert stdout == \
        "Open STEP file '{path}'...\n" \
        "Load model...\n" \
        "SUCCESS\n".format(path=fp)
    assert code == 0


def test_invalid_file(cli):
    fp = cli.add_file('LICENSE.txt')
    code, stdout, stderr = cli.run('open-step', fp)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert len(stderr) > 10
    assert "Open STEP file '{path}'...\n".format(path=fp) in stdout
    assert "Finished with errors!\n" in stdout
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test creating local libraries with the library manager
"""


def test(librepcb, helpers):
    """
    Create new local library
    """
    with librepcb.open() as app:

        # Open library manager
        app.widget('controlPanelOpenLibraryManagerButton').click()
        assert app.widget('libraryManager').properties()['visible'] is True

        # Make sure there is only one entry ("New Library") in the libraries list
        library_count_before = 1
        library_list = app.widget('libraryManagerInstalledLibrariesList')
        helpers.wait_for_model_items_count(library_list, library_count_before,
                                           library_count_before)

        # Create new library
        app.widget('libraryManagerAddLibraryTabBar').set_current_tab(1)
        widget_properties = {
            ('NameEdit', 'text'): 'Local Library',
            ('DescriptionEdit', 'text'): 'Foo Bar',
            ('AuthorEdit', 'text'): 'Functional Test',
            ('UrlEdit', 'text'): '',
            ('VersionEdit', 'text'): '1.2.3',
            ('Cc0LicenseCheckbox', 'checked'): True,
        }
        for (widget, property), value in widget_properties.items():
            app.widget('libraryManagerCreateLocalLibrary' + widget).set_property(property, value)
        app.widget('libraryManagerCreateLocalLibraryCreateButton').click()

        # Check if one library is added
        library_count_after = library_count_before + 1
        helpers.wait_for_model_items_count(library_list, library_count_after,
                                           library_count_after)

        # Open the new library to check if everything is OK
        app.widget('libraryManagerLibraryInfoWidgetOpenEditorButton').click()
        assert app.widget('libraryEditor').properties()['visible'] is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test creating projects
"""

import os


def test_new_project_wizard(librepcb):
    """
    Create project using the wizard from the control panel
    """
    with librepcb.open() as app:
        # Open new project wizard
        app.widget('controlPanelNewProjectButton').click()
        assert app.widget('controlPanelNewProjectWizard').properties()['visible'] is True
        # Enter metadata
        name = 'New Project'
        app.widget('controlPanelNewProjectWizardMetadataNameEdit').set_property('text', name)
        path = app.widget('controlPanelNewProjectWizardMetadataPathEdit').properties()['text']
        app.widget('controlPanelNewProjectWizardNextButton').click()
        # Setup schematic/board
        app.widget('controlPanelNewProjectWizardFinishButton').click()
        # Verify if editors are opened and project file exists
        assert app.widget('schematicEditor').properties()['visible'] is True
        assert app.widget('boardEditor').properties()['visible'] is True
        assert os.path.exists(path)

    # Open project again to see if it was saved properly
    librepcb.set_project(path)
    with librepcb.open() as app:
        assert app.widget('schematicEditor').properties()['visible'] is True
        assert app.widget('boardEditor').properties()['visible'] is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test copying a package category in the library editor
"""


def test(library_editor, helpers):
    """
    Copy package category with "New Library Element" wizard
    """
    le = library_editor

    # Open "New Library Element" wizard
    le.action('libraryEditorActionNewElement').trigger(blocking=False)

    # Choose "Copy existing element"
    le.widget('libraryEditorNewElementWizardChooseTypeCopyRadioButton').set_property('checked', True)

    # Choose type of element
    le.widget('libraryEditorNewElementWizardChooseTypePkgCatButton').click()

    # Choose category
    category_tree = le.widget('libraryEditorNewElementWizardCopyFromCategoriesTree')
    helpers.wait_for_model_items_count(category_tree, 4)
    category = category_tree.model().items().items[2]
    category_tree.select_item(category)
    le.widget('libraryEditorNewElementWizardNextButton').click()

    # Check metadata
    widget_properties = {
        # Note: C-SMT/Crystals are buggy due to Qt/ICU bug in sort algorithm!
        ('NameEdit', 'text'): 'R-SMT',
        ('DescriptionEdit', 'plainText'): '',
        ('KeywordsEdit', 'text'): '',
        ('VersionEdit', 'text'): '0.1',
    }
    for (widget, property), value in widget_properties.items():
        props = le.widget('libraryEditorNewElementWizardMetadata' + widget).properties()
        assert props[property] == value

    # Finish
    dialog = le.widget('libraryEditorNewElementWizard')
    le.widget('libraryEditorNewElementWizardFinishButton').click()
    helpers.wait_until_widget_hidden(dialog)

    # Check if a new tab is opened (indicates that the element was created)
    tab_props = le.widget('libraryEditorStackedWidget').properties()
    assert tab_props['count'] == 2
    assert tab_props['currentIndex'] == 1

    # Check metadata
    assert le.widget('libraryEditorPkgCatNameEdit').properties()['text'] == 'R-SMT'
    assert le.widget('libraryEditorPkgCatDescriptionEdit').properties()['plainText'] == ''
    assert le.widget('libraryEditorPkgCatKeywordsEdit').properties()['text'] == ''
    assert le.widget('libraryEditorPkgCatVersionEdit').properties()['text'] == '0.1'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import shutil
import pytest
import subprocess


CLI_DIR = os.path.dirname(__file__)
TESTS_DIR = os.path.dirname(CLI_DIR)
REPO_DIR = os.path.dirname(TESTS_DIR)
DATA_DIR = os.path.join(TESTS_DIR, 'data')


def pytest_addoption(parser):
    parser.addoption("--librepcb-executable",
                     action="store",
                     help="Path to librepcb-cli executable to test")


class CliExecutor(object):
    def __init__(self, config, tmpdir):
        super(CliExecutor, self).__init__()
        self.executable = os.path.abspath(config.getoption('--librepcb-executable'))
        if not os.path.exists(self.executable):
            raise Exception("Executable '{}' not found. Please pass it with "
                            "'--librepcb-executable'.".format(self.executable))
        self.tmpdir = tmpdir

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def abspath(self, relpath):
        return os.path.normpath(os.path.join(self.tmpdir, relpath))

    def add_library(self, library):
        src = os.path.join(DATA_DIR, 'libraries', library)
        dst = os.path.join(self.tmpdir, library)
        shutil.copytree(src, dst)

    def add_project(self, project, as_lppz=False):
        src = os.path.join(DATA_DIR, 'projects', project)
        dst = os.path.join(self.tmpdir, project)
        if as_lppz:
            shutil.make_archive(dst, 'zip', src)
            shutil.move(dst + '.zip', dst + '.lppz')
        else:
            shutil.copytree(src, dst)

    def add_file(self, relpath):
        src = os.path.join(DATA_DIR, relpath)
        dst = os.path.join(self.tmpdir, os.path.basename(relpath))
        shutil.copyfile(src, dst)
        return dst

    def run(self, *args):
        p = subprocess.Popen([self.executable] + list(args), cwd=self.tmpdir,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             universal_newlines=True, env=self._env())
        stdout, stderr = p.communicate()
        # output to stdout/stderr because it helps debugging failed tests
        sys.stdout.write(stdout)
        sys.stderr.write(stderr)
        return p.returncode, stdout, stderr

    def _env(self):
        env = os.environ
        # Make output independent from the system's language
        env['LC_ALL'] = 'C'
        # Override configuration location to make tests independent of existing configs
        env['LIBREPCB_CONFIG_DIR'] = os.path.join(self.tmpdir, 'config')
        # Use a neutral username
        env['USERNAME'] = 'testuser'
        # Disable warning about unstable file format, since tests are run also
        # on the (unstable) master branch
        env['LIBREPCB_DISABLE_UNSTABLE_WARNING'] = '1'
        return env


@pytest.fixture
def cli(request, tmpdir):
    """
    Fixture to start the LibrePCB CLI
    """
    with CliExecutor(request.config, str(tmpdir)) as executor:
        yield executor



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest

"""
Test command "open-step --minify"
"""


def test_valid_file(cli):
    fp = cli.add_file('unittests/librepcbcommon/OccModelTest/model.step')
    code, stdout, stderr = cli.run('open-step', '--minify', fp)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == ''
    assert stdout == \
        "Open STEP file '{path}'...\n" \
        "Perform minify...\n" \
        " - Minified from 28,521 bytes to 18,336 bytes (-36%)\n" \
        "Load model...\n" \
        "SUCCESS\n".format(path=fp)
    assert code == 0


def test_invalid_file(cli):
    fp = cli.add_file('LICENSE.txt')
    code, stdout, stderr = cli.run('open-step', '--minify', fp)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == 'ERROR: STEP data section not found.\n'
    assert stdout == \
        "Open STEP file '{path}'...\n" \
        "Perform minify...\n" \
        "Finished with errors!\n".format(path=fp)
    assert code == 1


def test_on_save_to_output(cli):
    fp = cli.add_file('unittests/librepcbcommon/OccModelTest/model.step')
    fp_out = cli.abspath('out.step')
    code, stdout, stderr = cli.run('open-step', '--minify',
                                   '--save-to', fp_out, fp)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == ''
    assert stdout == \
        "Open STEP file '{path}'...\n" \
        "Perform minify...\n" \
        " - Minified from 28,521 bytes to 18,336 bytes (-36%)\n" \
        "Save to '{outpath}'...\n" \
        "Load model...\n" \
        "SUCCESS\n".format(path=fp, outpath=fp_out)
    assert code == 0

    code, stdout, stderr = cli.run('open-step', '--minify', fp_out)
    assert stderr == ''
    assert stdout == \
        "Open STEP file '{path}'...\n" \
        "Perform minify...\n" \
        " - File is already minified\n" \
        "Load model...\n" \
        "SUCCESS\n".format(path=fp_out)
    assert code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

"""
Test command "open-project" (basic parser tests)
"""

HELP_TEXT = """\
Usage: {executable} [options] open-project [command_options] project
LibrePCB Command Line Interface

Options:
  -h, --help                         Print this message.
  -V, --version                      Displays version information.
  -v, --verbose                      Verbose output.
  --erc                              Run the electrical rule check, print all
                                     non-approved warnings/errors and report
                                     failure (exit code = 1) if there are
                                     non-approved messages.
  --drc                              Run the design rule check, print all
                                     non-approved warnings/errors and report
                                     failure (exit code = 1) if there are
                                     non-approved messages.
  --drc-settings <file>              Override DRC settings by providing a *.lp
                                     file containing custom settings. If not
                                     set, the settings from the boards will be
                                     used instead.
  --run-job <name>                   Run a particular output job. Can be given
                                     multiple times to run multiple jobs.
  --run-jobs                         Run all existing output jobs.
  --jobs <file>                      Override output jobs with a *.lp file
                                     containing custom jobs. If not set, the
                                     jobs from the project will be used instead.
  --outdir <path>                    Override the output base directory of
                                     jobs. If not set, the standard output
                                     directory from the project is used.
  --export-schematics <file>         Export schematics to given file(s).
                                     Existing files will be overwritten.
                                     Supported file extensions: pdf, svg, ***
  --export-bom <file>                Export generic BOM to given file(s).
                                     Existing files will be overwritten.
                                     Supported file extensions: csv
  --export-board-bom <file>          Export board-specific BOM to given
                                     file(s). Existing files will be
                                     overwritten. Supported file extensions: csv
  --bom-attributes <attributes>      Comma-separated list of additional
                                     attributes to be exported to the BOM.
                                     Example: "SUPPLIER, SKU"
  --export-pcb-fabrication-data      Export PCB fabrication data
                                     (Gerber/Excellon) according the fabrication
                                     output settings of boards. Existing files
                                     will be overwritten.
  --pcb-fabrication-settings <file>  Override PCB fabrication output settings
                                     by providing a *.lp file containing custom
                                     settings. If not set, the settings from the
                                     boards will be used instead.
  --export-pnp-top <file>            Export pick&place file for automated
                                     assembly of the top board side. Existing
                                     files will be overwritten. Supported file
                                     extensions: csv, gbr
  --export-pnp-bottom <file>         Export pick&place file for automated
                                     assembly of the bottom board side. Existing
                                     files will be overwritten. Supported file
                                     extensions: csv, gbr
  --export-netlist <file>            Export netlist file for automated PCB
                                     testing. Existing files will be
                                     overwritten. Supported file extensions:
                                     d356
  --board <name>                     The name of the board(s) to export. Can be
                                     given multiple times. If not set, all
                                     boards are exported.
  --board-index <index>              Same as '--board', but allows to specify
                                     boards by index instead of by name.
  --remove-other-boards              Remove all boards not specified with
                                     '--board[-index]' from the project before
                                     executing all the other actions. If
                                     '--board[-index]' is not passed, all boards
                                     will be removed. Pass '--save' to save the
                                     modified project to disk.
  --variant <name>                   The name of the assembly variant(s) to
                                     export. Can be given multiple times. If not
                                     set, all assembly variants are exported.
  --variant-index <index>            Same as '--variant', but allows to specify
                                     assembly variants by index instead of by
                                     name.
  --set-default-variant <name>       Move the specified assembly variant to the
                                     top before executing all the other actions.
                                     Pass '--save' to save the modified project
                                     to disk.
  --save                             Save project before closing it (useful to
                                     upgrade file format).
  --strict                           Fail if the project files are not strictly
                                     canonical, i.e. there would be changes when
                                     saving the project. Note that this option
                                     is not available for *.lppz files.

Arguments:
  open-project                       Open a project to execute project-related
                                     tasks.
  project                            Path to project file (*.lpp[z]).
"""

ERROR_TEXT = """\
{error}
Usage: {executable} [options] open-project [command_options] project
Help: {executable} open-project --help
"""


def _clean(help_text):
    """
    Remove client-dependent image file extensions from the help text to make
    the tests portable.
    """
    return re.sub(
        'Supported file extensions: pdf, svg,([\\s\\n]*[0-9a-z,]+)+',
        'Supported file extensions: pdf, svg, ***',
        help_text,
    )


def test_help(cli):
    code, stdout, stderr = cli.run('open-project', '--help')
    assert stderr == ''
    assert _clean(stdout) == HELP_TEXT.format(executable=cli.executable)
    assert code == 0


def test_no_arguments(cli):
    code, stdout, stderr = cli.run('open-project')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Missing arguments: project",
    )
    assert stdout == ''
    assert code == 1


def test_invalid_argument(cli):
    code, stdout, stderr = cli.run('open-project', '--invalid-argument')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Unknown option 'invalid-argument'.",
    )
    assert stdout == ''
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest

"""
Test if window settings (position, size, toolbar visibility, ...) are saved and
restored properly.
"""


def close_application(app):
    pass  # nothing to do, it will be closed automatically


def close_project_action(app):
    app.action('schematicEditorActionCloseProject').trigger()


def close_schematic_editor_first(app):
    app.widget('schematicEditor').close()
    app.widget('boardEditor').close()


def close_board_editor_first(app):
    app.widget('boardEditor').close()
    app.widget('schematicEditor').close()


@pytest.mark.parametrize('close_method', [
    close_application,
    close_project_action,
    close_schematic_editor_first,
    close_board_editor_first,
])
def test(librepcb, close_method):
    """
    Check settings with multiple variants how the windows can be closed:
      - Whole application closed by the OS
      - Project closed with "Close Project" tool button
      - Schematic and board editor windows closed, in both orders

    This is needed to check if settings are saved/restored properly in all
    cases.
    """
    new_sch_pos = (50, 60)
    new_sch_size = (810, 610)
    new_brd_pos = (70, 80)
    new_brd_size = (820, 620)

    librepcb.add_project('Empty Project')
    librepcb.set_project('Empty Project/Empty Project.lpp')
    with librepcb.open() as app:
        # check default settings (LibrePCB.ini did not exist yet)
        assert app.widget('schematicEditorToolbarFile').properties()['visible'] is True
        assert app.widget('schematicEditorDockErcMsg').properties()['visible'] is True
        assert app.widget('boardEditorToolbarFile').properties()['visible'] is True
        assert app.widget('boardEditorDockErcMsg').properties()['visible'] is True

        # change some settings
        new_sch_pos = app.widget('schematicEditor').move(*new_sch_pos)
        new_sch_size = app.widget('schematicEditor').resize(*new_sch_size)
        new_brd_pos = app.widget('boardEditor').move(*new_brd_pos)
        new_brd_size = app.widget('boardEditor').resize(*new_brd_size)

        # close application with specified method
        close_method(app)

    # restart librepcb
    with librepcb.open() as app:
        # check settings
        assert app.widget('schematicEditorToolbarFile').properties()['visible'] is True
        assert app.widget('schematicEditorDockErcMsg').properties()['visible'] is True
        assert app.widget('boardEditorToolbarFile').properties()['visible'] is True
        assert app.widget('boardEditorDockErcMsg').properties()['visible'] is True

        # Note: Window positions aren't checked because the window manager
        # might place the window on another position and thus that test would
        # not be reliable.

        props = app.widget('schematicEditor').properties()
        # assert (props['x'], props['y']) == new_sch_pos
        assert (props['width'], props['height']) == new_sch_size

        props = app.widget('boardEditor').properties()
        # assert (props['x'], props['y']) == new_brd_pos
        assert (props['width'], props['height']) == new_brd_size

        # hide all toolbars and docks
        app.widget('schematicEditorToolbarFile').set_properties(visible=False)
        app.widget('schematicEditorDockErcMsg').set_properties(visible=False)
        app.widget('boardEditorToolbarFile').set_properties(visible=False)
        app.widget('boardEditorDockErcMsg').set_properties(visible=False)

        # close application with specified method
        close_method(app)

    # restart librepcb
    with librepcb.open() as app:
        # check if all toolbars and docks are hidden
        assert app.widget('schematicEditorToolbarFile', wait_active=False).properties()['visible'] is False
        assert app.widget('schematicEditorDockErcMsg', wait_active=False).properties()['visible'] is False
        assert app.widget('boardEditorToolbarFile', wait_active=False).properties()['visible'] is False
        assert app.widget('boardEditorDockErcMsg', wait_active=False).properties()['visible'] is False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import params
import pytest

"""
Test command "open-project"
"""


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_open_project_absolute_path(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', cli.abspath(project.path))
    assert stderr == ''
    assert stdout == \
        "Open project '{path}'...\n" \
        "SUCCESS\n".format(path=cli.abspath(project.path))
    assert code == 0


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_open_project_relative_path(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_open_project_verbose(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', '--verbose', project.path)
    assert len(stderr) > 100  # logging messages are on stderr
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test the "Initialize Workspace Wizard" in the Control Panel to switch to
a different workspace
"""

import os
import shutil


def test_create_new_workspace(librepcb, helpers):
    """
    Choose a non-existent workspace path to create a new workspace
    """
    test_workspace_path = librepcb.workspace_path + '-test'
    with librepcb.open() as app:
        # Open "Switch Workspace" dialog.
        app.widget('controlPanel').properties()['visible'] is True
        app.action('controlPanelActionSwitchWorkspace').trigger(blocking=False)
        wizard = app.widget('controlPanelInitWorkspaceWizard')

        # Choose workspace path.
        path_edit = app.widget('controlPanelInitWorkspaceWizardChooseWorkspacePathEdit')
        assert path_edit.properties()['text'] == librepcb.workspace_path
        status_label = app.widget('controlPanelInitWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'contains a valid workspace' in status_label.properties()['text']
        path_edit.set_property('text', test_workspace_path)
        assert 'workspace will be created' in status_label.properties()['text']
        app.widget('controlPanelInitWorkspaceWizardNextButton').click()

        # Choose workspace settings.
        user_edit = app.widget('controlPanelInitWorkspaceWizardChooseSettingsUserNameEdit')
        user_edit.set_property('text', 'foobar 1337')
        app.widget('controlPanelInitWorkspaceWizardFinishButton').click()

        # Verify that the workspace has been created.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(os.path.join(test_workspace_path,
                                           '.librepcb-workspace'))

    # Open LibrePCB again to see if the workspace is automatically opened
    # and the settings are applied.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True
        app.action('controlPanelActionOpenWorkspaceSettings').trigger(blocking=False)
        user_edit = app.widget('controlPanelWorkspaceSettingsDialogGeneralUserNameEdit')
        assert user_edit.properties()['text'] == 'foobar 1337'


def test_open_compatible_workspace(librepcb, helpers):
    """
    Choose a compatible workspace to open
    """
    test_workspace_path = librepcb.workspace_path + '-test'
    shutil.copytree(librepcb.workspace_path, test_workspace_path)
    with librepcb.open() as app:
        # Open "Switch Workspace" dialog.
        app.widget('controlPanel').properties()['visible'] is True
        app.action('controlPanelActionSwitchWorkspace').trigger(blocking=False)
        wizard = app.widget('controlPanelInitWorkspaceWizard')

        # Choose workspace path.
        path_edit = app.widget('controlPanelInitWorkspaceWizardChooseWorkspacePathEdit')
        assert path_edit.properties()['text'] == librepcb.workspace_path
        status_label = app.widget('controlPanelInitWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'contains a valid workspace' in status_label.properties()['text']
        path_edit.set_property('text', test_workspace_path)
        assert 'contains a valid workspace' in status_label.properties()['text']
        app.widget('controlPanelInitWorkspaceWizardFinishButton').click()
        helpers.wait_until_widget_hidden(wizard)

    # Open LibrePCB again to see if the workspace is automatically opened.
    shutil.rmtree(librepcb.workspace_path)
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test "--help"

Note: It might look a bit picky to compare the CLI output with *exact* values,
but in the end it's very easy to update the expected values after each change
and it heavily increases test coverage.
"""

HELP_TEXT = """\
Usage: {executable} [options] command
LibrePCB Command Line Interface

Options:
  -h, --help     Print this message.
  -V, --version  Displays version information.
  -v, --verbose  Verbose output.

Arguments:
  command        The command to execute (see list below).

Commands:
  open-library   Open a library to execute library-related tasks.
  open-project   Open a project to execute project-related tasks.
  open-step      Open a STEP model to execute STEP-related tasks outside of a library.

List command-specific options:
  {executable} <command> --help
"""

ERROR_TEXT = """\
{error}
Usage: {executable} [options] command
Help: {executable} --help
"""


def test_explicit(cli):
    code, stdout, stderr = cli.run('--help')
    assert stderr == ''
    assert stdout == HELP_TEXT.format(executable=cli.executable)
    assert code == 0


def test_implicit_if_no_arguments(cli):
    code, stdout, stderr = cli.run()
    assert stderr == ''
    assert stdout == HELP_TEXT.format(executable=cli.executable)
    assert code == 0


def test_implicit_if_passing_invalid_command(cli):
    code, stdout, stderr = cli.run('invalid-command')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Unknown command 'invalid-command'.",
    )
    assert stdout == ''
    assert code == 1


def test_implicit_if_passing_invalid_argument(cli):
    code, stdout, stderr = cli.run('--invalid-argument')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Unknown option 'invalid-argument'.",
    )
    assert stdout == ''
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
import re

"""
Test command "open-step --tesselate"
"""


PATTERN = "Open STEP file '{path}'...\\n" \
          "Load model...\\n" \
          "Tesselate model...\\n" \
          " - Built \\d\\d+ vertices with [1-9]\\d* different colors\\n" \
          "SUCCESS\\n"


def test_valid_file(cli):
    fp = cli.add_file('unittests/librepcbcommon/OccModelTest/model.step')
    code, stdout, stderr = cli.run('open-step', '--tesselate', fp)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == ''
    assert re.fullmatch(PATTERN.format(path=re.escape(fp)), stdout)
    assert code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test opening the project editor
"""


def test_window_titles(project_editor):
    """
    Check window titles of schematic and board editor
    """
    se = project_editor.widget('schematicEditor')
    be = project_editor.widget('boardEditor')
    assert se.properties()['windowTitle'] == 'Empty Project.lpp - LibrePCB Schematic Editor'
    assert be.properties()['windowTitle'] == 'Empty Project.lpp - LibrePCB Board Editor'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project --save"
"""


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_save(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # append some zeros to the project file
    path = cli.abspath(project.path)
    with open(path, 'ab') as f:
        f.write(b'\0\0')
    original_filesize = os.path.getsize(path)
    # save project (must remove the appended zeros)
    code, stdout, stderr = cli.run('open-project', '--save', path)
    assert stderr == ''
    assert stdout == \
        "Open project '{path}'...\n" \
        "Save project...\n" \
        "SUCCESS\n".format(path=path)
    assert code == 0
    assert os.path.getsize(path) != original_filesize



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test opening projects by command line arguments
"""


def test_open_lpp(librepcb, helpers):
    """
    Open *.lpp project by command line argument
    """
    librepcb.add_project('Empty Project')
    librepcb.set_project('Empty Project/Empty Project.lpp')
    with librepcb.open() as app:
        # Check if both editors were opened
        assert app.widget('schematicEditor').properties()['visible'] is True
        assert app.widget('boardEditor').properties()['visible'] is True

        # Check if the schematic editor is the active window
        helpers.wait_for_active_window(app, app.widget('schematicEditor'))  # raises on timeout


def test_open_lppz(librepcb, helpers):
    """
    Open *.lppz project by command line argument
    """
    librepcb.add_project('Empty Project', as_lppz=True)
    librepcb.set_project('Empty Project.lppz')
    with librepcb.open() as app:
        # Check if both editors were opened
        assert app.widget('schematicEditor').properties()['visible'] is True
        assert app.widget('boardEditor').properties()['visible'] is True

        # Check if the schematic editor is the active window
        helpers.wait_for_active_window(app, app.widget('schematicEditor'))  # raises on timeout



from conans import ConanFile
from conans.tools import download, untargz
import os

class TypeSafe(ConanFile):
    name = 'type_safe'
    url  = 'https://github.com/foonathan/type_safe'
    version = '0.2.2'
    requires = 'debug_assert/1.3@Manu343726/testing'
    exports = '*.hpp'
    generators = 'cmake'


    def source(self):
        tar = 'type_safe-{}.tar.gz'.format(self.version)
        url = 'https://github.com/foonathan/type_safe/archive/v{}.tar.gz'.format(self.version)
        download(url, tar)
        untargz(tar)

    def package(self):
        includedir = os.path.join('include', 'type_safe')
        srcdir = os.path.join('type_safe-{}'.format(self.version), includedir)
        self.copy('*.hpp', src=srcdir, dst=includedir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import fileinput
import params
import pytest

"""
Test command "open-project --export-pcb-fabrication-data"
"""


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_if_project_without_boards_succeeds(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # remove all boards first
    with open(cli.abspath(project.dir + '/boards/boards.lp'), 'w') as f:
        f.write('(librepcb_boards)')

    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0
    assert not os.path.exists(dir)  # nothing was exported ;)


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.EMPTY_PROJECT_LPPZ_PARAM,
])
def test_export_project_with_one_board_implicit(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'default':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 9


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.EMPTY_PROJECT_LPPZ_PARAM,
])
def test_export_project_with_one_board_explicit(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--board=default',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'default':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 9


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_if_exporting_invalid_board_fails(cli, project):
    """
    Note: Test with passing the argument as "--arg <value>".
    """
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--board', 'foo',
                                   project.path)
    assert stderr == "ERROR: No board with the name 'foo' found.\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1
    assert not os.path.exists(dir)


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_export_project_with_two_boards_implicit(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'default':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr'\n" \
        "  Board 'copy':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SILKSCREEN-BOTTOM.gbr'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 18


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_export_project_with_two_boards_explicit_one(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--board=copy',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'copy':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SILKSCREEN-BOTTOM.gbr'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 9


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_export_project_with_two_boards_explicit_two(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--board=copy',
                                   '--board=default',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'copy':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_copy_SILKSCREEN-BOTTOM.gbr'\n" \
        "  Board 'default':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 18


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_export_project_with_two_conflicting_boards_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # change gerber output path to the same for both boards
    boardfile = cli.abspath(project.dir + '/boards/copy/board.lp')
    for line in fileinput.input(boardfile, inplace=1):
        print(line.replace("_copy", ""))

    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   project.path)
    assert stderr == \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr' was written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr' was written multiple times!\n" \
        "NOTE: To avoid writing files multiple times, make sure to pass " \
        "unique filepaths to all export functions. For board output files, " \
        "you could either add the placeholder '{{{{BOARD}}}}' to the path or " \
        "specify the boards to export with the '--board' argument.\n"\
        .format(project=project).replace('//', os.sep)
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'default':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr'\n" \
        "  Board 'copy':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr'\n" \
        "Finished with errors!\n".format(project=project).replace('//', os.sep)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_export_project_with_two_conflicting_boards_succeeds_explicit(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # change gerber output path to the same for both boards
    boardfile = cli.abspath(project.dir + '/boards/copy/board.lp')
    for line in fileinput.input(boardfile, inplace=1):
        print(line.replace("_copy", ""))

    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--board=copy',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'copy':\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-NPTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_DRILLS-PTH.drl'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_OUTLINES.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_COPPER-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-TOP.gbr'\n" \
        "    => '{project.output_dir_native}//gerber//Empty_Project_SILKSCREEN-BOTTOM.gbr'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 9


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.EMPTY_PROJECT_LPPZ_PARAM,
])
def test_export_with_custom_settings(cli, project):
    """
    Notes:
      - Test with passing the argument as "--arg <value>".
      - Test attributes '{{BOARD_INDEX}}' and '{{BOARD_DIRNAME}}' in path.
    """
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    settings = """
      (fabrication_output_settings
        (base_path "./out/{{BOARD_INDEX}}_{{BOARD_DIRNAME}}/")
        (outlines (suffix "OUTLINES.gbr"))
        (copper_top (suffix "COPPER-TOP.gbr"))
        (copper_inner (suffix "COPPER-IN{{CU_LAYER}}.gbr"))
        (copper_bot (suffix "COPPER-BOTTOM.gbr"))
        (soldermask_top (suffix "SOLDERMASK-TOP.gbr"))
        (soldermask_bot (suffix "SOLDERMASK-BOTTOM.gbr"))
        (silkscreen_top (suffix "SILKSCREEN-TOP.gbr")
          (layers top_placement top_names)
        )
        (silkscreen_bot (suffix "SILKSCREEN-BOTTOM.gbr")
          (layers bot_placement bot_names)
        )
        (drills (merge true)
          (suffix_pth "DRILLS-PTH.drl")
          (suffix_npth "DRILLS-NPTH.drl")
          (suffix_merged "DRILLS.drl")
          (suffix_buried "_DRILLS-{{START_LAYER}}-{{END_LAYER}}.drl")
          (g85_slots false)
        )
        (solderpaste_top (create true) (suffix "SOLDERPASTE-TOP.gbr"))
        (solderpaste_bot (create true) (suffix "SOLDERPASTE-BOTTOM.gbr"))
      )
    """
    with open(cli.abspath('settings.lp'), mode='w') as f:
        f.write(settings)
    dir = os.path.dirname(cli.abspath(project.path)) + '/out/0_default'
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--pcb-fabrication-settings', 'settings.lp',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "  Board 'default':\n" \
        "    => '{output_prefix}out//0_default//DRILLS.drl'\n" \
        "    => '{output_prefix}out//0_default//OUTLINES.gbr'\n" \
        "    => '{output_prefix}out//0_default//COPPER-TOP.gbr'\n" \
        "    => '{output_prefix}out//0_default//COPPER-BOTTOM.gbr'\n" \
        "    => '{output_prefix}out//0_default//SOLDERMASK-TOP.gbr'\n" \
        "    => '{output_prefix}out//0_default//SOLDERMASK-BOTTOM.gbr'\n" \
        "    => '{output_prefix}out//0_default//SILKSCREEN-TOP.gbr'\n" \
        "    => '{output_prefix}out//0_default//SILKSCREEN-BOTTOM.gbr'\n" \
        "    => '{output_prefix}out//0_default//SOLDERPASTE-TOP.gbr'\n" \
        "    => '{output_prefix}out//0_default//SOLDERPASTE-BOTTOM.gbr'\n" \
        "SUCCESS\n".format(
            project=project,
            output_prefix=('' if project.is_lppz else (project.dir + os.sep)),
        ).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 10


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_if_export_with_nonexistent_settings_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--pcb-fabrication-settings=nonexistent.lp',
                                   project.path)
    assert stderr == \
        "ERROR: Failed to load custom settings: " \
        "The file \"{file}\" does not exist.\n" \
        .format(file=cli.abspath('nonexistent.lp'))
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1
    assert not os.path.exists(dir)


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_if_export_with_invalid_settings_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    with open(cli.abspath('settings.lp'), mode='w') as f:
        f.write('foobar')
    dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pcb-fabrication-data',
                                   '--pcb-fabrication-settings=settings.lp',
                                   project.path)
    assert stderr == \
        "ERROR: Failed to load custom settings: File parse error: " \
        "Child not found: base_path/@0\n" \
        "File: \n" \
        "Invalid Content: ''\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export PCB fabrication data...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1
    assert not os.path.exists(dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import params

"""
Test command "open-project --strict"
"""


def test_valid_lpp(cli):
    project = params.EMPTY_PROJECT_LPP
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', '--strict', project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Check for non-canonical files...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


def test_invalid_lpp(cli):
    project = params.EMPTY_PROJECT_LPP
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # append some zeros to the project file
    path = cli.abspath(project.path)
    with open(path, 'ab') as f:
        f.write(b'\0\0')
    # open project
    code, stdout, stderr = cli.run('open-project', '--strict', project.path)
    assert stderr == \
        "    - Non-canonical file: '{project.path}'\n" \
        .format(project=project)
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Check for non-canonical files...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1


def test_lppz_fails(cli):
    project = params.PROJECT_WITH_TWO_BOARDS_LPPZ
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', '--strict', project.path)
    assert stderr == \
        "  ERROR: The option '--strict' is not available for *.lppz files!\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Check for non-canonical files...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import params
import pytest

"""
Test command "open-project --drc"
"""


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_if_project_without_boards_succeeds(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # remove all boards
    with open(cli.abspath(project.dir + '/boards/boards.lp'), 'w') as f:
        f.write('(librepcb_boards)')
    code, stdout, stderr = cli.run('open-project', '--drc', project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run DRC...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM])
def test_project_with_two_boards_explicit_one(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', '--drc', '--board=copy',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run DRC...\n" \
        "  Board 'copy':\n" \
        "    Approved messages: 0\n" \
        "    Non-approved messages: 0\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_board_with_approved_message(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # Add a hole with small diameter and the corresponding approval.
    with open(cli.abspath(project.dir + '/boards/default/board.lp'), 'r') as f:
        board_content = f.read()
    board_content = board_content.replace(
        '\n)\n',
        '\n (hole 82506db2-3323-4732-8480-f3517f10dd44 '
        '  (diameter 0.1) (stop_mask auto) (lock false)\n'
        '  (vertex (position 50.0 50.0) (angle 0.0))\n'
        ' )\n'
        ')\n'
    )
    board_content = board_content.replace(
        ' (design_rule_check\n',
        ' (design_rule_check\n'
        '  (approved minimum_drill_diameter_violation\n'
        '   (hole 82506db2-3323-4732-8480-f3517f10dd44)\n'
        '  )\n'
    )
    with open(cli.abspath(project.dir + '/boards/default/board.lp'), 'w') as f:
        f.write(board_content)
    code, stdout, stderr = cli.run('open-project', '--drc', project.path)
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run DRC...\n" \
        "  Board 'default':\n" \
        "    Approved messages: 1\n" \
        "    Non-approved messages: 0\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_board_without_messages(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', '--drc', project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run DRC...\n" \
        "  Board 'default':\n" \
        "    Approved messages: 0\n" \
        "    Non-approved messages: 0\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_board_with_nonapproved_message(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # Add a hole with small diameter.
    with open(cli.abspath(project.dir + '/boards/default/board.lp'), 'r') as f:
        board_content = f.read()
    board_content = board_content.replace(
        '\n)\n',
        '\n (hole 82506db2-3323-4732-8480-f3517f10dd44 '
        '  (diameter 0.1) (stop_mask auto) (lock false)\n'
        '  (vertex (position 50.0 50.0) (angle 0.0))\n'
        ' )\n'
        ')\n'
    )
    with open(cli.abspath(project.dir + '/boards/default/board.lp'), 'w') as f:
        f.write(board_content)
    code, stdout, stderr = cli.run('open-project', '--drc', project.path)
    assert stderr == "      - [WARNING] NPTH drill diameter: 0.1 < 0.25 mm\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run DRC...\n" \
        "  Board 'default':\n" \
        "    Approved messages: 0\n" \
        "    Non-approved messages: 1\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_with_custom_settings(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    settings = """
      (design_rule_check
       (min_copper_copper_clearance 0.2)
       (min_copper_board_clearance 0.3)
       (min_copper_npth_clearance 0.2)
       (min_drill_drill_clearance 0.35)
       (min_drill_board_clearance 0.5)
       (min_silkscreen_stopmask_clearance 0.127)
       (min_copper_width 0.2)
       (min_annular_ring 0.15)
       (min_npth_drill_diameter 0.1234)
       (min_pth_drill_diameter 0.25)
       (min_npth_slot_width 1.0)
       (min_pth_slot_width 0.7)
       (min_silkscreen_width 0.1)
       (min_silkscreen_text_height 0.8)
       (min_outline_tool_diameter 2.0)
       (blind_vias_allowed true)
       (buried_vias_allowed true)
       (allowed_npth_slots single_segment_straight)
       (allowed_pth_slots single_segment_straight)
       (approvals_version "0.2")
      )
    """
    with open(cli.abspath('settings.lp'), mode='w') as f:
        f.write(settings)
    # Add a hole with small diameter.
    with open(cli.abspath(project.dir + '/boards/default/board.lp'), 'r') as f:
        board_content = f.read()
    board_content = board_content.replace(
        '\n)\n',
        '\n (hole 82506db2-3323-4732-8480-f3517f10dd44 '
        '  (diameter 0.1) (stop_mask auto) (lock false)\n'
        '  (vertex (position 50.0 50.0) (angle 0.0))\n'
        ' )\n'
        ')\n'
    )
    with open(cli.abspath(project.dir + '/boards/default/board.lp'), 'w') as f:
        f.write(board_content)
    code, stdout, stderr = cli.run('open-project',
                                   '--drc',
                                   '--drc-settings', 'settings.lp',
                                   project.path)
    assert stderr == "      - [WARNING] NPTH drill diameter: 0.1 < 0.1234 mm\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run DRC...\n" \
        "  Board 'default':\n" \
        "    Approved messages: 0\n" \
        "    Non-approved messages: 1\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import params
import pytest

"""
Test command "open-library"
"""


@pytest.mark.parametrize("library", [
    params.EMPTY_LIBRARY_PARAM,
    params.POPULATED_LIBRARY_PARAM,
])
def test_open_library_absolute_path(cli, library):
    cli.add_library(library.dir)
    code, stdout, stderr = cli.run('open-library', cli.abspath(library.dir))
    assert stderr == ''
    assert stdout == \
        "Open library '{dir}'...\n" \
        "SUCCESS\n".format(dir=cli.abspath(library.dir))
    assert code == 0


@pytest.mark.parametrize("library", [
    params.EMPTY_LIBRARY_PARAM,
    params.POPULATED_LIBRARY_PARAM,
])
def test_open_library_relative_path(cli, library):
    cli.add_library(library.dir)
    code, stdout, stderr = cli.run('open-library', library.dir)
    assert stderr == ''
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "SUCCESS\n".format(library=library)
    assert code == 0


@pytest.mark.parametrize("library", [
    params.EMPTY_LIBRARY_PARAM,
    params.POPULATED_LIBRARY_PARAM,
])
def test_open_library_all(cli, library):
    cli.add_library(library.dir)
    code, stdout, stderr = cli.run('open-library', '--all', library.dir)
    assert stderr == ''
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "SUCCESS\n".format(library=library)
    assert code == 0


@pytest.mark.parametrize("library", [
    params.POPULATED_LIBRARY_PARAM,
])
def test_open_library_all_verbose(cli, library):
    cli.add_library(library.dir)
    code, stdout, stderr = cli.run('open-library', '--all', '--verbose',
                                   library.dir)
    assert len(stderr) > 100  # logging messages are on stderr
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "SUCCESS\n".format(library=library)
    assert code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

"""
Test "--version"
"""

PATTERN = "LibrePCB CLI Version \\d+\\.\\d+\\.\\d+(\\-\\w+)?\\n" \
          "File Format [0-9]+(\\.[0-9]+)? \\((stable|unstable)\\)\\n" \
          "Git Revision [0-9a-f]+\\n" \
          "Qt Version [0-9\\.]+ \\(compiled against [0-9\\.]+\\)\\n" \
          "OpenCascade [A-Z/]+( [0-9\\.]+)?\\n" \
          "Built at [0-9a-zA-Z\\.:/ ]+\\n"


def test_valid_call(cli):
    code, stdout, stderr = cli.run('--version')
    assert stderr == ''
    assert re.fullmatch(PATTERN, stdout)
    assert code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project --export-pnp-top --export-pnp-bottom"
"""


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
@pytest.mark.parametrize("argument,side", [
    ('--export-pnp-top=foo.bar', 'top'),
    ('--export-pnp-bottom=foo.bar', 'bottom'),
])
def test_if_unknown_file_extension_fails(cli, project, argument, side):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project', argument, project.path)
    assert stderr == "  ERROR: Unknown extension 'bar'.\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export {side} assembly data to 'foo.bar'...\n" \
        "  - 'default' => 'foo.bar'\n" \
        "Finished with errors!\n" \
        .format(project=project, side=side)
    assert code == 1


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
@pytest.mark.parametrize("ext", ['csv', 'gbr'])
def test_if_project_without_boards_succeeds(cli, project, ext):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # remove all boards first
    with open(cli.abspath(project.dir + '/boards/boards.lp'), 'w') as f:
        f.write('(librepcb_boards)')

    relpath_top = project.output_dir + '/pnp/top.' + ext
    relpath_bot = project.output_dir + '/pnp/bot.' + ext
    abspath_top = cli.abspath(relpath_top)
    abspath_bot = cli.abspath(relpath_bot)
    assert not os.path.exists(abspath_top)
    assert not os.path.exists(abspath_bot)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pnp-top=' + relpath_top,
                                   '--export-pnp-bottom=' + relpath_bot,
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export top assembly data to '{project.output_dir}/pnp/top.{ext}'...\n" \
        "Export bottom assembly data to '{project.output_dir}/pnp/bot.{ext}'...\n" \
        "SUCCESS\n".format(project=project, ext=ext)
    assert code == 0
    assert not os.path.exists(abspath_top)  # nothing exported
    assert not os.path.exists(abspath_bot)  # nothing exported


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
@pytest.mark.parametrize("ext", ['csv', 'gbr'])
def test_export_project_with_two_boards_implicit(cli, project, ext):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    fp_top = project.output_dir + '/pnp/{{BOARD}}_top.' + ext
    fp_bot = project.output_dir + '/pnp/{{BOARD}}_bot.' + ext
    dir = cli.abspath(project.output_dir + '/pnp')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pnp-top=' + fp_top,  # --arg="val"
                                   '--export-pnp-bottom', fp_bot,  # --arg "val"
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export top assembly data to '{project.output_dir}/pnp/{{{{BOARD}}}}_top.{ext}'...\n" \
        "  - 'default' => '{project.output_dir_native}//pnp//default_top.{ext}'\n" \
        "  - 'copy' => '{project.output_dir_native}//pnp//copy_top.{ext}'\n" \
        "Export bottom assembly data to '{project.output_dir}/pnp/{{{{BOARD}}}}_bot.{ext}'...\n" \
        "  - 'default' => '{project.output_dir_native}//pnp//default_bot.{ext}'\n" \
        "  - 'copy' => '{project.output_dir_native}//pnp//copy_bot.{ext}'\n" \
        "SUCCESS\n".format(project=project, ext=ext).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 4


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
@pytest.mark.parametrize("ext", ['csv', 'gbr'])
def test_export_project_with_two_boards_explicit_one(cli, project, ext):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    fp_top = project.output_dir + '/pnp/{{BOARD}}_top.' + ext
    fp_bot = project.output_dir + '/pnp/{{BOARD}}_bot.' + ext
    dir = cli.abspath(project.output_dir + '/pnp')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pnp-top=' + fp_top,
                                   '--export-pnp-bottom=' + fp_bot,
                                   '--board=copy',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export top assembly data to '{project.output_dir}/pnp/{{{{BOARD}}}}_top.{ext}'...\n" \
        "  - 'copy' => '{project.output_dir_native}//pnp//copy_top.{ext}'\n" \
        "Export bottom assembly data to '{project.output_dir}/pnp/{{{{BOARD}}}}_bot.{ext}'...\n" \
        "  - 'copy' => '{project.output_dir_native}//pnp//copy_bot.{ext}'\n" \
        "SUCCESS\n".format(project=project, ext=ext).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 2


@pytest.mark.parametrize("project", [params.PROJECT_WITH_TWO_BOARDS_LPP])
@pytest.mark.parametrize("ext", ['csv', 'gbr'])
def test_export_project_with_two_conflicting_boards_fails(cli, project, ext):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    fp_top = project.output_dir + '/top.' + ext
    fp_bot = project.output_dir + '/bot.' + ext
    code, stdout, stderr = cli.run('open-project',
                                   '--export-pnp-top=' + fp_top,
                                   '--export-pnp-bottom=' + fp_bot,
                                   project.path)
    assert stderr == \
        "ERROR: The file '{project.output_dir_native}//bot.{ext}' was " \
        "written multiple times!\n" \
        "ERROR: The file '{project.output_dir_native}//top.{ext}' was " \
        "written multiple times!\n" \
        "NOTE: To avoid writing files multiple times, make sure to pass " \
        "unique filepaths to all export functions. For board output files, " \
        "you could either add the placeholder '{{{{BOARD}}}}' to the path or " \
        "specify the boards to export with the '--board' argument.\n" \
        .format(project=project, ext=ext).replace('//', os.sep)
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export top assembly data to '{project.output_dir}/top.{ext}'...\n" \
        "  - 'default' => '{project.output_dir_native}//top.{ext}'\n" \
        "  - 'copy' => '{project.output_dir_native}//top.{ext}'\n" \
        "Export bottom assembly data to '{project.output_dir}/bot.{ext}'...\n" \
        "  - 'default' => '{project.output_dir_native}//bot.{ext}'\n" \
        "  - 'copy' => '{project.output_dir_native}//bot.{ext}'\n" \
        "Finished with errors!\n" \
        .format(project=project, ext=ext).replace('//', os.sep)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
import sys

"""
Test the overview tab in the library editor
"""


def list_dclick_helper(le, widget_name, number):
    # Double click on the list must do nothing
    widget = le.widget(widget_name)
    widget.dclick()
    assert le.widget('libraryEditorStackedWidget').properties()['count'] == number
    assert le.widget('libraryEditorStackedWidget').properties()['currentIndex'] == 0

    # Double click on an item must open the element in a new tab
    widget.dclick_item(widget.model().items().items[0])
    assert le.widget('libraryEditorStackedWidget').properties()['count'] == number + 1
    assert le.widget('libraryEditorStackedWidget').properties()['currentIndex'] == number

    # Switch back to overview tab
    le.widget('libraryEditorTabBar').set_current_tab(0)


def test_edit_with_doubleclick(library_editor, helpers):
    """
    Open editor tabs with doubleclick in the elements list
    """
    list_dclick_helper(library_editor, 'libraryEditorOverviewCmpCatList', 1)
    list_dclick_helper(library_editor, 'libraryEditorOverviewPkgCatList', 2)
    list_dclick_helper(library_editor, 'libraryEditorOverviewSymList', 3)
    list_dclick_helper(library_editor, 'libraryEditorOverviewPkgList', 4)
    list_dclick_helper(library_editor, 'libraryEditorOverviewCmpList', 5)
    list_dclick_helper(library_editor, 'libraryEditorOverviewDevList', 6)


def delete_shortcut_helper(le, widget_name, valid, tabs):
    tabwidget = le.widget('libraryEditorStackedWidget')
    widget = le.widget(widget_name)
    widget.activate_focus()
    if valid:
        # Select first item in the list
        widget.click_item(widget.model().items().items[0])
    widget.shortcut('Del')
    if valid:
        le.widget('libraryEditorOverviewMsgBoxBtnCancel').click()
        tabwidget.wait_for_properties({'count': tabs + 1, 'currentIndex': 0})
    widget.shortcut('Del')
    if valid:
        le.widget('libraryEditorOverviewMsgBoxBtnYes').click()
    tabwidget.wait_for_properties({'count': tabs, 'currentIndex': 0})


# This test is flaky on Windows CI which already caused lots of frustration.
# So let's allow it to fail on Windows. By testing the feature manually on
# Windows, there was no wrong behavior so it might be a false-positive.
@pytest.mark.xfail(sys.platform == "win32", reason="Flaky test on Windows.")
def test_delete_shortcut(library_editor, helpers):
    """
    Test removing library elements with the "delete" keyboard shortcut
    """
    le = library_editor

    # Open some tabs first
    test_edit_with_doubleclick(library_editor, helpers)
    assert le.widget('libraryEditorStackedWidget').properties()['count'] == 7

    # Press delete while the focus is not in a list widget
    delete_shortcut_helper(le, 'libraryEditorOverviewNameEdit', False, 7)

    # Delete all selected elements
    delete_shortcut_helper(le, 'libraryEditorOverviewCmpCatList', True, 6)
    delete_shortcut_helper(le, 'libraryEditorOverviewPkgCatList', True, 5)
    delete_shortcut_helper(le, 'libraryEditorOverviewSymList', True, 4)
    delete_shortcut_helper(le, 'libraryEditorOverviewPkgList', True, 3)
    delete_shortcut_helper(le, 'libraryEditorOverviewCmpList', True, 2)
    delete_shortcut_helper(le, 'libraryEditorOverviewDevList', True, 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test opening the library editor
"""


def test_metadata(library_editor):
    """
    Just open the library editor and check some metadata
    """
    le = library_editor
    assert le.widget('libraryEditorStackedWidget').properties()['count'] == 1
    assert le.widget('libraryEditorOverviewNameEdit').properties()['text'] == 'Populated Library'
    assert le.widget('libraryEditorOverviewDescriptionEdit').properties()['plainText'] == 'Test library containing some elements'
    assert le.widget('libraryEditorOverviewKeywordsEdit').properties()['text'] == 'test,library,populated'
    assert le.widget('libraryEditorOverviewAuthorEdit').properties()['text'] == 'test'
    assert le.widget('libraryEditorOverviewVersionEdit').properties()['text'] == '0.0.1'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test the "Initialize Workspace Wizard" on an existing installation, i.e.
with a workspace already set in the user settings
"""

import os
import pytest
import shutil
import sys


@pytest.mark.xfail(sys.platform == 'darwin', raises=AssertionError,
                   strict=False, reason="Flaky on macOS, no idea why.")
def test_upgrade_v01(librepcb, helpers):
    """
    Test an upgrade which copies from 'v0.1' to 'data'
    """
    v01_path = os.path.join(librepcb.workspace_path, 'v0.1')
    data_path = os.path.join(librepcb.workspace_path, 'data')
    shutil.rmtree(data_path)
    with librepcb.open() as app:
        # Perform upgrade.
        wizard = app.widget('initWorkspaceWizard')
        src_label = app.widget('initWorkspaceWizardUpgradeSourceLabel')
        assert v01_path in src_label.properties()['text']
        dst_label = app.widget('initWorkspaceWizardUpgradeDestinationLabel')
        assert data_path in dst_label.properties()['text']
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the data directory has been created.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(os.path.join(data_path, '.librepcb-data'))

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True


@pytest.mark.xfail(sys.platform == 'darwin', raises=AssertionError,
                   strict=False, reason="Flaky on macOS, no idea why.")
def test_upgrade_data(librepcb, helpers):
    """
    Test an upgrade which copies from 'data' to 'v0.1'
    """
    v01_path = os.path.join(librepcb.workspace_path, 'v0.1')
    data_path = os.path.join(librepcb.workspace_path, 'data')
    shutil.rmtree(v01_path)
    os.remove(os.path.join(data_path, '.librepcb-data'))  # Downgrade to v0.1
    with librepcb.open() as app:
        # Perform upgrade.
        wizard = app.widget('initWorkspaceWizard')
        src_label = app.widget('initWorkspaceWizardUpgradeSourceLabel')
        assert data_path in src_label.properties()['text']
        dst_label = app.widget('initWorkspaceWizardUpgradeDestinationLabel')
        assert v01_path in dst_label.properties()['text']
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the v0.1 directory has been created and the data
        # directory has been upgraded.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(v01_path)
        assert os.path.exists(os.path.join(data_path, '.librepcb-data'))

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True


def test_downgrade(librepcb, helpers):
    """
    Test opening a workspace which needs to be initialized with an
    older file format
    """
    v01_path = os.path.join(librepcb.workspace_path, 'v0.1')
    v02_path = os.path.join(librepcb.workspace_path, 'v1')
    data_path = os.path.join(librepcb.workspace_path, 'data')
    shutil.rmtree(v01_path)
    with open(os.path.join(data_path, '.librepcb-data'), 'wb') as f:
        f.write(b'999\n')
    with librepcb.open() as app:
        # Choose workspace settings.
        wizard = app.widget('initWorkspaceWizard')
        user_edit = app.widget('initWorkspaceWizardChooseSettingsUserNameEdit')
        user_edit.set_property('text', 'foobar 42')
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the v1 directory has been created.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(os.path.join(v02_path, '.librepcb-data'))

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import pytest


class Library:
    def __init__(self, dir, cmpcat, pkgcat, sym, pkg, cmp, dev):
        self.dir = dir
        self.cmpcat = cmpcat
        self.pkgcat = pkgcat
        self.sym = sym
        self.pkg = pkg
        self.cmp = cmp
        self.dev = dev


EMPTY_LIBRARY = Library('Empty Library.lplib', 0, 0, 0, 0, 0, 0)
EMPTY_LIBRARY_PARAM = pytest.param(EMPTY_LIBRARY,
                                   id='EmptyLibrary')

POPULATED_LIBRARY = Library('Populated Library.lplib', 10, 4, 19, 8, 15, 8)
POPULATED_LIBRARY_PARAM = pytest.param(POPULATED_LIBRARY,
                                       id='PopulatedLibrary')


class Project:
    def __init__(self, dir, path, output_dir, board_count):
        self.dir = dir
        self.path = os.path.normpath(path)
        self.parent_dir = os.path.dirname(path)
        self.output_dir = output_dir
        self.output_dir_native = os.path.normpath(output_dir)
        self.is_lppz = path.endswith('.lppz')
        self.board_count = board_count


EMPTY_PROJECT_LPP = Project(
    'Empty Project',
    'Empty Project/Empty Project.lpp',
    'Empty Project/output/v1',
    1,
)
EMPTY_PROJECT_LPP_PARAM = pytest.param(EMPTY_PROJECT_LPP,
                                       id='EmptyProject.lpp')

EMPTY_PROJECT_LPPZ = Project(
    'Empty Project',
    'Empty Project.lppz',
    'output/v1',
    1,
)
EMPTY_PROJECT_LPPZ_PARAM = pytest.param(EMPTY_PROJECT_LPPZ,
                                        id='EmptyProject.lppz')

PROJECT_WITH_TWO_BOARDS_LPP = Project(
    'Project With Two Boards',
    'Project With Two Boards/Project With Two Boards.lpp',
    'Project With Two Boards/output/v1',
    2,
)
PROJECT_WITH_TWO_BOARDS_LPP_PARAM = pytest.param(PROJECT_WITH_TWO_BOARDS_LPP,
                                                 id='ProjectWithTwoBoards.lpp')

PROJECT_WITH_TWO_BOARDS_LPPZ = Project(
    'Project With Two Boards',
    'Project With Two Boards.lppz',
    'output/v1',
    2,
)
PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM = pytest.param(PROJECT_WITH_TWO_BOARDS_LPPZ,
                                                  id='ProjectWithTwoBoards.lppz')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import fileinput
import params
import pytest

"""
Test command "open-project --run-jobs"
"""


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
])
def test_if_project_without_jobs_succeeds(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    dir = cli.abspath(project.output_dir)
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0
    assert os.path.exists(dir)
    assert os.listdir(dir) == ['.librepcb-output']


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_project_with_jobs(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath(project.output_dir)
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   project.path)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run output job 'Schematic PDF'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1_Schematic.pdf'\n" \
        "Run output job 'Board Assembly PDF'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1_Assembly.pdf'\n" \
        "Run output job 'Gerber/Excellon'...\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_DRILLS-NPTH.drl'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_DRILLS-PTH.drl'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_OUTLINES.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_COPPER-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_COPPER-BOTTOM.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERMASK-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERMASK-BOTTOM.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SILKSCREEN-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SILKSCREEN-BOTTOM.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERPASTE-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERPASTE-BOTTOM.gbr'\n" \
        "Run output job 'Pick&Place CSV'...\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_TOP.csv'\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_BOT.csv'\n" \
        "Run output job 'Pick&Place X3'...\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_TOP.gbr'\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_BOT.gbr'\n" \
        "Run output job 'Netlist'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1_Netlist.d356'\n" \
        "Run output job 'BOM'...\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_BOM_AV.csv'\n" \
        "Run output job 'STEP Model'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1.step'\n" \
        "Run output job 'Custom File'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1.txt'\n" \
        "Run output job 'ZIP'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1.zip'\n" \
        "Run output job 'Project Data'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1.json'\n" \
        "Run output job 'Project Archive'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1.lppz'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 11


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_if_invalid_job_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--run-job', 'foo',
                                   project.path)
    assert stderr == "ERROR: No output job with the name 'foo' found.\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_conflicting_output_file_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # change one output file to be identical as another one
    boardfile = cli.abspath(project.dir + '/project/jobs.lp')
    for line in fileinput.input(boardfile, inplace=1):
        print(line.replace("_TOP.gbr", "_BOT.gbr"))

    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   project.path)
    assert stderr == \
        "ERROR: Attempted to write the output file " \
        "'asm//Empty_Project_v1_PnP_AV_BOT.gbr' multiple times! " \
        "Make sure to specify unique output file paths, e.g. by using " \
        "placeholders like '{{BOARD}}' or '{{VARIANT}}'.\n" \
        .replace('//', os.sep)
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run output job 'Schematic PDF'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1_Schematic.pdf'\n" \
        "Run output job 'Board Assembly PDF'...\n" \
        "  => '{project.output_dir_native}//Empty_Project_v1_Assembly.pdf'\n" \
        "Run output job 'Gerber/Excellon'...\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_DRILLS-NPTH.drl'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_DRILLS-PTH.drl'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_OUTLINES.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_COPPER-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_COPPER-BOTTOM.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERMASK-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERMASK-BOTTOM.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SILKSCREEN-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SILKSCREEN-BOTTOM.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERPASTE-TOP.gbr'\n" \
        "  => '{project.output_dir_native}//gbr//Empty_Project_v1_SOLDERPASTE-BOTTOM.gbr'\n" \
        "Run output job 'Pick&Place CSV'...\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_TOP.csv'\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_BOT.csv'\n" \
        "Run output job 'Pick&Place X3'...\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_BOT.gbr'\n" \
        "  => '{project.output_dir_native}//asm//Empty_Project_v1_PnP_AV_BOT.gbr'\n" \
        "Finished with errors!\n".format(project=project).replace('//', os.sep)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.EMPTY_PROJECT_LPPZ_PARAM,
])
def test_custom_jobs(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    jobs = """
      (librepcb_jobs
       (job a334a18d-6bf7-4e99-b48e-a26c2e2899bd (name "Custom Job")
        (type netlist)
        (board default)
        (output "custom.d356")
       )
      )
    """
    with open(cli.abspath('custom_jobs.lp'), mode='w') as f:
        f.write(jobs)
    dir = cli.abspath(project.output_dir)
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   '--jobs', 'custom_jobs.lp',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run output job 'Custom Job'...\n" \
        "  => '{output_prefix}output//v1//custom.d356'\n" \
        "SUCCESS\n".format(
            project=project,
            output_prefix=('' if project.is_lppz else (project.dir + os.sep)),
        ).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 2


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
])
def test_nonexistent_jobs_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   '--jobs=nonexistent.lp',
                                   project.path)
    assert stderr == \
        "ERROR: Failed to load custom output jobs: " \
        "The file \"{file}\" does not exist.\n" \
        .format(file=cli.abspath('nonexistent.lp'))
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
])
def test_invalid_jobs_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    with open(cli.abspath('custom_jobs.lp'), mode='w') as f:
        f.write('(librepcb_jobs (job))')
    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   '--jobs=custom_jobs.lp',
                                   project.path)
    assert stderr == \
        "ERROR: Failed to load custom output jobs: File parse error: " \
        "Child not found: type/@0\n" \
        "File: \n" \
        "Invalid Content: ''\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_custom_outdir_relative(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath('foo')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   '--outdir=foo',
                                   project.path)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run output job 'Schematic PDF'...\n" \
        "  => 'foo//Empty_Project_v1_Schematic.pdf'\n" \
        "Run output job 'Board Assembly PDF'...\n" \
        "  => 'foo//Empty_Project_v1_Assembly.pdf'\n" \
        "Run output job 'Gerber/Excellon'...\n" \
        "  => 'foo//gbr//Empty_Project_v1_DRILLS-NPTH.drl'\n" \
        "  => 'foo//gbr//Empty_Project_v1_DRILLS-PTH.drl'\n" \
        "  => 'foo//gbr//Empty_Project_v1_OUTLINES.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_COPPER-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_COPPER-BOTTOM.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERMASK-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERMASK-BOTTOM.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SILKSCREEN-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SILKSCREEN-BOTTOM.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERPASTE-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERPASTE-BOTTOM.gbr'\n" \
        "Run output job 'Pick&Place CSV'...\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_TOP.csv'\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_BOT.csv'\n" \
        "Run output job 'Pick&Place X3'...\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_TOP.gbr'\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_BOT.gbr'\n" \
        "Run output job 'Netlist'...\n" \
        "  => 'foo//Empty_Project_v1_Netlist.d356'\n" \
        "Run output job 'BOM'...\n" \
        "  => 'foo//asm//Empty_Project_v1_BOM_AV.csv'\n" \
        "Run output job 'STEP Model'...\n" \
        "  => 'foo//Empty_Project_v1.step'\n" \
        "Run output job 'Custom File'...\n" \
        "  => 'foo//Empty_Project_v1.txt'\n" \
        "Run output job 'ZIP'...\n" \
        "  => 'foo//Empty_Project_v1.zip'\n" \
        "Run output job 'Project Data'...\n" \
        "  => 'foo//Empty_Project_v1.json'\n" \
        "Run output job 'Project Archive'...\n" \
        "  => 'foo//Empty_Project_v1.lppz'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 11


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_custom_outdir_absolute(cli, project):
    """
    Note: Actually the stdout should show absolute paths as well, but that's
    not implemented correctly yet.
    """
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath('foo')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--run-jobs',
                                   '--outdir', dir,
                                   project.path)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run output job 'Schematic PDF'...\n" \
        "  => 'foo//Empty_Project_v1_Schematic.pdf'\n" \
        "Run output job 'Board Assembly PDF'...\n" \
        "  => 'foo//Empty_Project_v1_Assembly.pdf'\n" \
        "Run output job 'Gerber/Excellon'...\n" \
        "  => 'foo//gbr//Empty_Project_v1_DRILLS-NPTH.drl'\n" \
        "  => 'foo//gbr//Empty_Project_v1_DRILLS-PTH.drl'\n" \
        "  => 'foo//gbr//Empty_Project_v1_OUTLINES.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_COPPER-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_COPPER-BOTTOM.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERMASK-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERMASK-BOTTOM.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SILKSCREEN-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SILKSCREEN-BOTTOM.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERPASTE-TOP.gbr'\n" \
        "  => 'foo//gbr//Empty_Project_v1_SOLDERPASTE-BOTTOM.gbr'\n" \
        "Run output job 'Pick&Place CSV'...\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_TOP.csv'\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_BOT.csv'\n" \
        "Run output job 'Pick&Place X3'...\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_TOP.gbr'\n" \
        "  => 'foo//asm//Empty_Project_v1_PnP_AV_BOT.gbr'\n" \
        "Run output job 'Netlist'...\n" \
        "  => 'foo//Empty_Project_v1_Netlist.d356'\n" \
        "Run output job 'BOM'...\n" \
        "  => 'foo//asm//Empty_Project_v1_BOM_AV.csv'\n" \
        "Run output job 'STEP Model'...\n" \
        "  => 'foo//Empty_Project_v1.step'\n" \
        "Run output job 'Custom File'...\n" \
        "  => 'foo//Empty_Project_v1.txt'\n" \
        "Run output job 'ZIP'...\n" \
        "  => 'foo//Empty_Project_v1.zip'\n" \
        "Run output job 'Project Data'...\n" \
        "  => 'foo//Empty_Project_v1.json'\n" \
        "Run output job 'Project Archive'...\n" \
        "  => 'foo//Empty_Project_v1.lppz'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 11



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test installing remote libraries with the library manager
"""


def test(librepcb, helpers):
    """
    Install some remote libraries
    """
    with librepcb.open() as app:

        # Open library manager
        app.widget('controlPanelOpenLibraryManagerButton').click()
        assert app.widget('libraryManager').properties()['visible'] is True

        # Make sure there is only one entry ("New Library") in the libraries list
        library_count_before = 1
        library_list = app.widget('libraryManagerInstalledLibrariesList')
        helpers.wait_for_model_items_count(library_list, library_count_before,
                                           library_count_before)

        # Wait until all libraries are fetched and check the count of them
        remote_library_count = 3  # The dummy API provides 3 libraries
        remote_library_list = app.widget('libraryManagerInstallLibrariesLibraryList')
        helpers.wait_for_model_items_count(remote_library_list,
                                           remote_library_count,
                                           remote_library_count)

        # Get the required widgets of all libraries and check their initial state
        statuslabels = list()
        checkboxes = list()
        progressbars = list()
        for i in range(0, remote_library_count):
            suffix = "-{}".format(i) if i > 0 else ''
            item_path = app.aliases['libraryManagerInstallLibrariesLibraryListItem'] + suffix
            statuslabel = app.widget(path=item_path + '::lblInstalledVersion')
            assert statuslabel.properties()['text'] == 'Recommended'
            statuslabels.append(statuslabel)
            checkbox = app.widget(path=item_path + '::cbxDownload')
            assert checkbox.properties()['checked'] is True  # Recommended
            checkboxes.append(checkbox)
            progressbar = app.widget(path=item_path + '::prgProgress', wait_active=False)
            assert progressbar.properties()['value'] == 0
            progressbars.append(progressbar)

        # Deselect first library to install -> this must also uncheck the
        # other libraries because it's a dependency!
        checkboxes[0].set_property('checked', False)

        # Verify checked state of all libraries
        for i in range(0, remote_library_count):
            assert checkboxes[i].properties()['checked'] is False

        # Select second library to install -> this must also check the first
        # library because it's a dependency!
        checkboxes[1].set_property('checked', True)

        # Verify checked state of all libraries
        for i in range(0, remote_library_count):
            assert checkboxes[i].properties()['checked'] is (True if i <= 1 else False)

        # Install selected libraries
        app.widget('libraryManagerInstallLibrariesDownloadButton').click()

        # Check if two libraries were added
        library_count_after = library_count_before + 2
        helpers.wait_for_model_items_count(library_list, library_count_after,
                                           library_count_after)

        # Wait until progress bars are at 100% and hidden (i.e. installation finished)
        for i in range(0, remote_library_count):
            props = {'value': 100 if i <= 1 else 0, 'visible': False}
            progressbars[i].wait_for_properties(props=props)

        # Check installed status of libraries in remote library list
        for i in range(0, remote_library_count):
            if i <= 1:
                # The downloaded libraries are outdated, thus the installed
                # version number is displayed.
                assert statuslabels[i].properties()['text'].startswith('v')
            else:
                assert statuslabels[i].properties()['text'] == 'Recommended'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project --export-bom"
"""


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_if_project_without_boards_succeeds(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # remove all boards first
    with open(cli.abspath(project.dir + '/boards/boards.lp'), 'w') as f:
        f.write('(librepcb_boards)')

    relpath = project.output_dir + '/bom/bom.csv'
    abspath = cli.abspath(relpath)
    assert not os.path.exists(abspath)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-bom=' + relpath,
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export generic BOM to '{project.output_dir}/bom/bom.csv'...\n" \
        "  => '{project.output_dir_native}//bom//bom.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(abspath)


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_export_multiple_files(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    relpath1 = project.output_dir + '/bom1.csv'
    abspath1 = cli.abspath(relpath1)
    assert not os.path.exists(abspath1)
    relpath2 = project.output_dir + '/bom2.csv'
    abspath2 = cli.abspath(relpath2)
    assert not os.path.exists(abspath2)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-bom=' + relpath1,  # --arg="value"
                                   '--export-bom', relpath2,  # --arg "value"
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export generic BOM to '{project.output_dir}/bom1.csv'...\n" \
        "  => '{project.output_dir_native}//bom1.csv'\n" \
        "Export generic BOM to '{project.output_dir}/bom2.csv'...\n" \
        "  => '{project.output_dir_native}//bom2.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(abspath1)
    assert os.path.exists(abspath2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import params
import pytest

"""
Test command "open-project --erc"
"""


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_project_with_approved_message(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # Add a unused net.
    with open(cli.abspath(project.dir + '/circuit/circuit.lp'), 'w') as f:
        f.write(
            '(librepcb_circuit'
            ' (variant 2511a923-d9f8-4cd0-940b-346185fbe32d (name "AV0")'
            '  (description "")'
            ' )'
            ' (netclass ada6400e-cb5c-41b6-9a4e-e2f39ffa3383 (name "C"))'
            ' (net 0654411b-a090-4025-a026-4c61e686662e (auto false) (name "N")'
            '  (netclass ada6400e-cb5c-41b6-9a4e-e2f39ffa3383)'
            ' )'
            ')'
        )
    # Approve the resulting message.
    with open(cli.abspath(project.dir + '/circuit/erc.lp'), 'w') as f:
        f.write(
            '(librepcb_erc'
            ' (approved open_net (net 0654411b-a090-4025-a026-4c61e686662e))'
            ')'
        )
    code, stdout, stderr = cli.run('open-project', '--erc', project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run ERC...\n" \
        "  Approved messages: 1\n" \
        "  Non-approved messages: 0\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_project_without_messages(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # Make sure there are no approved messages.
    with open(cli.abspath(project.dir + '/circuit/erc.lp'), 'w') as f:
        f.write('(librepcb_erc)')
    # now the actual test...
    code, stdout, stderr = cli.run('open-project', '--erc', project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run ERC...\n" \
        "  Approved messages: 0\n" \
        "  Non-approved messages: 0\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_project_with_nonapproved_message(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # Add a unused net.
    with open(cli.abspath(project.dir + '/circuit/circuit.lp'), 'w') as f:
        f.write(
            '(librepcb_circuit'
            ' (variant 2511a923-d9f8-4cd0-940b-346185fbe32d (name "AV0")'
            '  (description "")'
            ' )'
            ' (netclass ada6400e-cb5c-41b6-9a4e-e2f39ffa3383 (name "C"))'
            ' (net 0654411b-a090-4025-a026-4c61e686662e (auto false) (name "N")'
            '  (netclass ada6400e-cb5c-41b6-9a4e-e2f39ffa3383)'
            ' )'
            ')'
        )
    # Make sure there are no approved messages.
    with open(cli.abspath(project.dir + '/circuit/erc.lp'), 'w') as f:
        f.write('(librepcb_erc)')
    # now the actual test...
    code, stdout, stderr = cli.run('open-project', '--erc', project.path)
    assert stderr == "    - [WARNING] Less than two pins in net: 'N'\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Run ERC...\n" \
        "  Approved messages: 0\n" \
        "  Non-approved messages: 1\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import time
import platform
import shutil
import pytest
import funq
from copy import deepcopy
from funq.client import ApplicationContext, ApplicationConfig
from funq.errors import FunqError


# Avoid errors due to too long file path on Windows :-|
def _long_path(path):
    if os.name == 'nt':
        return '\\\\?\\' + path.replace('/', '\\')
    else:
        return path


FUNQ_DIR = os.path.dirname(__file__)
TESTS_DIR = os.path.dirname(FUNQ_DIR)
REPO_DIR = os.path.dirname(TESTS_DIR)
DATA_DIR = os.path.join(TESTS_DIR, 'data')


def pytest_addoption(parser):
    parser.addoption("--librepcb-executable",
                     action="store",
                     help="Path to librepcb executable to test")


class GlobalOptions:
    def __init__(self):
        self.funq_conf = 'funq.conf'
        self.funq_attach_exe = funq.tools.which('funq')
        self.funq_gkit = 'default'
        self.funq_gkit_file = os.path.join(os.path.dirname(os.path.realpath(funq.client.__file__)), 'aliases-gkits.conf')


class Application(object):
    def __init__(self, executable, env=None, args=()):
        super(Application, self).__init__()
        cfg = ApplicationConfig(executable=executable, args=args, cwd=os.getcwd(), env=env,
                                aliases=os.path.join(FUNQ_DIR, 'aliases'), global_options=GlobalOptions())
        self._context = ApplicationContext(cfg)

    def __enter__(self):
        return self._context.funq

    def __exit__(self, exc_type, exc_val, exc_tb):
        del self._context


class LibrePcbFixture(object):
    def __init__(self, config, tmpdir):
        super(LibrePcbFixture, self).__init__()
        self.executable = os.path.abspath(config.getoption('--librepcb-executable'))
        if not os.path.exists(self.executable):
            raise Exception("Executable '{}' not found. Please pass it with "
                            "'--librepcb-executable'.".format(self.executable))
        self.tmpdir = tmpdir
        # Copy test data to temporary directory to avoid modifications in original data
        shutil.copytree(_long_path(os.path.join(DATA_DIR, 'workspaces', 'Empty Workspace')),
                        _long_path(os.path.join(self.tmpdir, 'Empty Workspace')))
        # Init members to default values
        self.workspace_path = os.path.join(self.tmpdir, 'Empty Workspace')
        self.project_path = None

        # Set environment variables
        self.env = deepcopy(os.environ)
        # Make GUI independent from the system's language
        self.env['LC_ALL'] = 'C'
        # Override configuration location to make tests independent of existing configs
        self.env['LIBREPCB_CONFIG_DIR'] = os.path.join(self.tmpdir, 'config')
        # Override cache location to make each test indepotent
        self.env['LIBREPCB_CACHE_DIR'] = os.path.join(self.tmpdir, 'cache')
        # Use a neutral username
        self.env['USERNAME'] = 'testuser'
        # Force LibrePCB to use Qt-style file dialogs because native dialogs don't work
        self.env['LIBREPCB_DISABLE_NATIVE_DIALOGS'] = '1'
        # Disable warning about unstable file format, since tests are run also
        # on the (unstable) master branch
        self.env['LIBREPCB_DISABLE_UNSTABLE_WARNING'] = '1'

    def abspath(self, relpath):
        return os.path.join(self.tmpdir, relpath)

    def get_data_path(self, relpath):
        return os.path.join(DATA_DIR, relpath)

    def set_workspace(self, path):
        if not os.path.isabs(path):
            path = self.abspath(path)
        self.workspace_path = path

    def add_project(self, project, as_lppz=False):
        src = os.path.join(DATA_DIR, 'projects', project)
        dst = os.path.join(self.tmpdir, project)
        if as_lppz:
            shutil.make_archive(dst, 'zip', src)
            shutil.move(dst + '.zip', dst + '.lppz')
        else:
            shutil.copytree(_long_path(src), _long_path(dst))

    def set_project(self, path):
        if not os.path.isabs(path):
            path = self.abspath(path)
        self.project_path = path

    def get_workspace_libraries_path(self, subdir=''):
        return os.path.join(self.workspace_path, 'data', 'libraries', subdir)

    def add_local_library_to_workspace(self, path):
        if not os.path.isabs(path):
            path = os.path.join(DATA_DIR, path)
        dest = self.get_workspace_libraries_path('local')
        dest = os.path.join(dest, os.path.basename(path))
        shutil.copytree(_long_path(path), _long_path(dest))

    def open(self):
        self._create_application_config_file()
        return Application(self.executable, env=self.env, args=self._args())

    def _create_application_config_file(self):
        org_dir = 'LibrePCB.org' if platform.system() == 'Darwin' else 'LibrePCB'
        config_dir = os.path.join(self.tmpdir, 'config', org_dir)
        config_ini = os.path.join(config_dir, 'LibrePCB.ini')
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)
        # Only create config file once per test, so tests can check if settings
        # are stored permanently.
        if not os.path.exists(config_ini):
            with open(config_ini, 'w') as f:
                if self.workspace_path:
                    f.write("[workspaces]\n")
                    f.write("most_recently_used=\"{}\"\n".format(self.workspace_path.replace('\\', '/')))

    def _args(self):
        args = []
        if self.project_path:
            args.append(self.project_path)
        return args


class Helpers(object):
    @staticmethod
    def wait_for_model_items_count(widget, min_count, max_count=None, timeout=5.0):
        if max_count == 0:
            # First wait a bit to be sure the model is really not populated
            time.sleep(0.1)
        count = None
        for i in range(0, 100):
            count = len(widget.model().items().items)
            if min_count <= count and (max_count is None or count <= max_count):
                return
            time.sleep(timeout / 100.0)
        raise Exception('Widget "{}" has {} items instead of [{}..{}]!'.format(
            widget.properties().get('objectName'), count, min_count, max_count))

    @staticmethod
    def wait_for_library_scan_complete(app, timeout=10.0):
        progress_bar = app.widget('controlPanelStatusBarProgressBar', wait_active=False)
        # wait until scan has started (progress > 10%)
        for i in range(0, 100):
            percent = progress_bar.properties()['value']
            if percent > 10:
                break
            time.sleep(timeout / 100.0)
        # Wait until scan has finished (progressbar hidden)
        Helpers.wait_until_widget_hidden(progress_bar, timeout=timeout)

    @staticmethod
    def wait_until_widget_hidden(widget, timeout=5.0):
        for i in range(0, 100):
            try:
                if widget.properties()['visible'] is False:
                    return
            except FunqError as e:
                if e.classname == 'NotRegisteredObject':
                    return
                raise
            time.sleep(timeout / 100.0)
        raise Exception('Widget "{}" is still visible!'.format(
            widget.properties().get('objectName')))

    @staticmethod
    def wait_for_active_window(funq, widget, timeout=5.0):
        Helpers._wait_for_active_widget(funq, widget, timeout, 'window')

    @staticmethod
    def wait_for_active_dialog(funq, widget, timeout=5.0):
        Helpers._wait_for_active_widget(funq, widget, timeout, 'modal')

    @staticmethod
    def _wait_for_active_widget(funq, widget, timeout, widget_type):
        active_widget = None
        for i in range(0, 100):
            active_widget = funq.active_widget(widget_type=widget_type)
            if active_widget is not None and active_widget.oid == widget.oid:
                return
            time.sleep(timeout / 100.0)
        properties = active_widget.properties() if active_widget else dict()
        raise Exception('Active widget is "{}" ({})!'.format(
            properties.get('windowTitle'), properties.get('objectName')))


@pytest.fixture(scope="session")
def librepcb_server():
    """
    Fixture which provides a HTTP server at localhost:8080

    All tests should use this server instead of the official LibrePCB API server
    or GitHub for downloading libraries.
    """
    import time
    import threading
    import socket
    import socketserver
    import http.server

    class Handler(http.server.SimpleHTTPRequestHandler, object):
        def translate_path(self, path):
            path = super(Handler, self).translate_path(path)
            relpath = os.path.relpath(path, os.curdir)
            return os.path.join(DATA_DIR, 'server', relpath)

    # Set SO_REUSEADDR option to avoid "port already in use" errors
    httpd = socketserver.TCPServer(("", 50080), Handler, bind_and_activate=False)
    httpd.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    httpd.server_bind()
    httpd.server_activate()
    thread = threading.Thread(target=httpd.serve_forever)
    thread.daemon = True
    thread.start()
    time.sleep(0.2)  # wait a bit to make sure the server is ready


@pytest.fixture
def create_librepcb(request, tmpdir, librepcb_server):
    """
    Fixture allowing to create multiple application instances
    """
    def _create():
        return LibrePcbFixture(request.config, str(tmpdir))
    return _create


@pytest.fixture
def librepcb(create_librepcb):
    """
    Fixture allowing to create one application instance
    """
    yield create_librepcb()


@pytest.fixture(scope="session")
def helpers():
    """
    Fixture providing some helper functions
    """
    return Helpers()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project" with all available options
"""


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.EMPTY_PROJECT_LPPZ_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_everything(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # prepare "--export-schematics"
    schematic = cli.abspath('schematic.pdf')
    assert not os.path.exists(schematic)

    # prepare "--export-bom"
    bom = cli.abspath('bom.csv')
    assert not os.path.exists(bom)

    # prepare "--export-board-bom"
    board_bom_dir = cli.abspath(project.output_dir + '/bom')
    assert not os.path.exists(board_bom_dir)
    board_bom = board_bom_dir + r'/{{BOARD}}.csv'

    # prepare "--export-pnp-top" and "--export-pnp-bottom"
    pnp_csv_dir = cli.abspath(project.output_dir + '/pnp_csv')
    assert not os.path.exists(pnp_csv_dir)
    pnp_top = pnp_csv_dir + r'/{{BOARD}}.csv'
    pnp_gbr_dir = cli.abspath(project.output_dir + '/pnp_gbr')
    assert not os.path.exists(pnp_gbr_dir)
    pnp_bot = pnp_gbr_dir + r'/{{BOARD}}.gbr'

    # prepare "--export-pcb-fabrication-data"
    gerber_dir = cli.abspath(project.output_dir + '/gerber')
    assert not os.path.exists(gerber_dir)

    # prepare "--save"
    path = cli.abspath(project.path)
    with open(path, 'ab') as f:
        f.write(b'\0\0')
    original_filesize = os.path.getsize(path)

    # run the CLI
    code, stdout, stderr = cli.run('open-project',
                                   '--erc',
                                   '--export-schematics={}'.format(schematic),
                                   '--export-bom={}'.format(bom),
                                   '--export-board-bom={}'.format(board_bom),
                                   '--export-pnp-top={}'.format(pnp_top),
                                   '--export-pnp-bottom={}'.format(pnp_bot),
                                   '--export-pcb-fabrication-data',
                                   '--save',
                                   project.path)
    assert stderr == ''
    assert len(stdout) > 100
    assert stdout.endswith('SUCCESS\n')
    assert code == 0

    # check "--export-schematics"
    assert os.path.exists(schematic)

    # check "--export-bom"
    assert os.path.exists(bom)

    # check "--export-board-bom"
    assert os.path.exists(board_bom_dir)
    assert len(os.listdir(board_bom_dir)) == project.board_count

    # check "--export-pnp-top"
    assert os.path.exists(pnp_csv_dir)
    assert len(os.listdir(pnp_csv_dir)) == project.board_count

    # check "--export-pnp-bottom"
    assert os.path.exists(pnp_gbr_dir)
    assert len(os.listdir(pnp_gbr_dir)) == project.board_count

    # check "--export-pcb-fabrication-data"
    assert os.path.exists(gerber_dir)
    assert len(os.listdir(gerber_dir)) == project.board_count * 9

    # check "--save"
    assert os.path.getsize(path) != original_filesize
